#!/usr/bin/perl
# Karel Srot, xsrot@math.muni.cz
#
# skript hleda jubilea v souborech s nasledujici strukturou:
# nazev_dne den mesic rok zprava
# napr.:  Pa 13 * * Dnes je patek trinacteho.
#
# standardne se prochazi soubor "svatky.dat", soubor "~/.svatky" (pokud existuje) a dale pak nazvy soubory predane parametry
# kontroluje se dnesek, zitrek a pozitri
# radky zacinajici # jsou komentare
# program ma jedinou volbu: -d pocet_dni    implicitne 2, max. 99

use Getopt::Std;

&getopts("d:");

# zpracuju volbu
if ($opt_d =~ /[1-9]\d?/) {
  $pocet_dni = $opt_d;
} else {
  $pocet_dni = 2;
}

@specifikace_dne = ("Dnes", "Zitra", "Pozitri");
@nazvy_mesicu = ("ledna", "února", "března", "dubna", "května", "června", "července", "srpna", "září", "října", "listopadu", "prosince");
@zkratky_dni = ("Ne", "Po", "Ut", "St", "Ct", "Pa", "So");
%nazvy_dni = ("Ne", "nedělě", "Po", "pondělí", "Ut", "úterý", "St", "středa", "Ct", "ctvrtek", "Pa", "pátek", "So", "Sobota");
# nastavim nazvy souboru k prohledavani.
@soubory = ("svatky.dat");
# pokud existuje soubor "~/.svatky" a mohu jej cist, pridam i jej
if (-r "$ENV{'HOME'}/.svatky") {
  push(@soubory, "$ENV{'HOME'}/.svatky");
}
# nakonec pridam zbyle argumenty
push(@soubory, @ARGV);

# pro dva dny
for ($i=0; $i<$pocet_dni; $i++) {
  # zjistim jednotlive udaje
  @aktualni_udaje = &vratCasoveUdaje($i);
  $srovnavaci_retezec = "$aktualni_udaje[0]:$aktualni_udaje[1]:$aktualni_udaje[2]:$aktualni_udaje[3]";;

  # vypisi udaj o dni
  if ($i<@specifikace_dne) {
    if ($i == 0) {
      print "\n$specifikace_dne[$i] je $nazvy_dni{$aktualni_udaje[0]} $aktualni_udaje[1]. $nazvy_mesicu[$aktualni_udaje[2]-1] 20$aktualni_udaje[3]:\n**********\n";
    } else {
      print "\n$specifikace_dne[$i]:\n**********\n";
    }  
  } elsif ($i<5) {
    print "\nZa $i dny:\n**********\n";
  } else {
    print "\nZa $i dní:\n**********\n";
  }
  
  # prohledam kazdy soubor
  foreach $soubor (@soubory) {
    if (open(SOUB, $soubor)) {
      # budu kontrolovat kazdy radek souboru
      while ($radek = <SOUB>) {
        chomp($radek);
        # pokud nejde o komentar
        if ($radek !~ /^\#/) {
          # preberu udaje z radku
          if ($radek =~ /^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)/) {
            @udaje_ze_souboru = ($1, $2, $3, $4, $5);
            #print "@udaje_ze_souboru\n";
            # nahradim * aktualni hodnotou
            for ($j=0; $j<4; $j++) { 
              if ($udaje_ze_souboru[$j] =~ /\*/) {
                $udaje_ze_souboru[$j] = $aktualni_udaje[$j];
              } 
            }
            $srovnavaci_retezec_soub = "$udaje_ze_souboru[0]:$udaje_ze_souboru[1]:$udaje_ze_souboru[2]:$udaje_ze_souboru[3]";
            # print "$srovnavaci_retezec_soub-$srovnavaci_retezec-\n";
            # pokud vse sedi
            if ($srovnavaci_retezec_soub =~ /$srovnavaci_retezec/) {
              print "$udaje_ze_souboru[4]\n";
            } else {
              # print "neshoda!";
            }
          }
        }
      }
      close(SOUB);
    } else {
      # chybovou hlasku vypisu pro soubor jen pri prvnim otvirani, tedy $i=0
      if ($i == 0) {
        print STDERR "Nelze otevrit soubor $soubor!\n";
      }
    }
  }
}

# jeste odradkuju
print "\n";

# procedura vraci pole udaju o datu pro zadany den ode dneska (0=dnes)
# (nazev_dne, den, mesic, rok(2-ciferne))
sub vratCasoveUdaje {
  $posun = $_[0];
  $muj_cas = time();
  # posunu se pripadne o zadany pocet dni vpred
  if ($posun>0) {
    $muj_cas = $muj_cas + $posun*3600*24;
  }
  # zjistim polozky podle data
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($muj_cas);
  return ($zkratky_dni[$wday], $mday, $mon+1, sprintf("%02d", $year % 100));
}
